;;; display-operands -- args-fold program argument processor example
;;;
;;; Copyright (c) 2002 Anthony Carrico
;;;
;;; All rights reserved.
;;;
;;; Redistribution and use in source and binary forms, with or without
;;; modification, are permitted provided that the following conditions
;;; are met:
;;; 1. Redistributions of source code must retain the above copyright
;;;    notice, this list of conditions and the following disclaimer.
;;; 2. Redistributions in binary form must reproduce the above copyright
;;;    notice, this list of conditions and the following disclaimer in the
;;;    documentation and/or other materials provided with the distribution.
;;; 3. The name of the authors may not be used to endorse or promote products
;;;    derived from this software without specific prior written permission.
;;;
;;; THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
;;; IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
;;; OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
;;; IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
;;; INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
;;; NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
;;; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
;;; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;;; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
;;; THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(define help-option
  (option '(#\h "help")
          #f
          #f
          (lambda (option name arg operands)
            (display "Usage: display-operands [option ...] [operand ...]

Display one operand on each line then exit.

  -h --help    display this help and exit
  -V --version output version information and exit
")
            (exit 0))))

(define version-option
  (option '(#\V "version")
          #f
          #f
          (lambda (option name arg operands)
            (display "display-operands 0.1

Copyright (C) 2002 Anthony Carrico
See the file COPYING. There is NO warranty.
")
            (exit 0))))

(define display-operands
  (lambda (command-line-arguments)
    (let ((operands
           (reverse
            (args-fold
             command-line-arguments
             (list help-option version-option)
             (lambda (option name arg operands)
               (display "unrecognized command line argument\n"
                        (error-output-port))
               (exit 1))
             (lambda (operand operands)
               (cons operand operands))
             '()))))
      (for-each
       (lambda (operand)
         (display operand)
         (newline))
       operands))))
